#include <io.h>
#include <fcntl.h>

#include "xscommon.h"
#include "patchack.h"
/* only for NULL */
#include <stddef.h>

#define GET_OFFS(x) ((x) & 0x00FFFFFFUL)
#define GET_BYTE(x) ((x) >> 24)

int ph_open(char *name, DWORD size, struct ftime *ft) {
int fl;
  fl = -1;
  if (FileExists(name, size)) {
    fl = open(name, O_RDWR | O_BINARY);
    if ((ft) && (fl != -1)) {
      getftime(fl, ft);
    }
  }
  return(fl);
}

void ph_close(int *fl, struct ftime *ft) {
  if ((fl) && (*fl != -1)) {
    if (ft) {
      setftime(*fl, ft);
    }
    close(*fl);
    *fl = -1;
  }
}

BYTE ph_read(int fl, DWORD offs) {
BYTE b;
  b = 0;
  if (fl != -1) {
    lseek(fl, GET_OFFS(offs), SEEK_SET);
    read(fl, &b, 1);
  }
  return(b);
}

void ph_write(int fl, DWORD offs, BYTE b) {
  if (fl != -1) {
    lseek(fl, GET_OFFS(offs), SEEK_SET);
    write(fl, &b, 1);
  }
}

void LoadPatches(fix_file *flist, fix_info *ilist, WORD isize) {
BYTE fn, b;
int fl;
  fn = 0xFF;
  fl = -1;
  while (isize--) {
    /* different file */
    if (fn != ilist->file) {
      fn = ilist->file;
      /* close last if opened */
      ph_close(&fl, NULL);
      /* open new file */
      fl = ph_open(flist[fn].name, flist[fn].size, NULL);
    }
    /* by default - file not found */
    ilist->stat = PH_NON;
    /* if file exists */
    if (fl != -1) {
      /* lazy check - only first byte checked */
      b = ph_read(fl, ilist->list->offs);
      /* by default - unknown byte */
      ilist->stat = PH_DIF;
      /* modified */
      if (b == ilist->list->data) {
        ilist->stat = PH_MOD;
      }
      /* original */
      if (b == GET_BYTE(ilist->list->offs)) {
        ilist->stat = PH_ORG;
      }
    }
    /* next info item */
    ilist++;
  }
  /* close last file if any */
  ph_close(&fl, NULL);
}

void SavePatches(fix_file *flist, fix_info *ilist, WORD isize) {
struct ftime ft;
BYTE fn;
WORD i;
int fl;
  fn = 0xFF;
  fl = -1;
  while (isize--) {
    /* different file */
    if (fn != ilist->file) {
      fn = ilist->file;
      /* close last if opened */
      ph_close(&fl, &ft);
      /* open new file */
      fl = ph_open(flist[fn].name, flist[fn].size, &ft);
    }
    /* if file exists and allowed to edit */
    if ((fl != -1) && (ilist->stat != PH_NON)) {
      for (i = 0; i < ilist->size; i++) {
        ph_write(fl, ilist->list[i].offs,
          (ilist->stat == PH_ORG) ? GET_BYTE(ilist->list[i].offs) : ilist->list[i].data
        );
      }
    }
    /* next info item */
    ilist++;
  }
  /* close last file if any */
  ph_close(&fl, &ft);
}

/* read actual data patches */
void ReadPatches(fix_file *flist, fix_info *ilist, WORD isize) {
BYTE fn;
WORD i;
int fl;
  fn = 0xFF;
  fl = -1;
  while (isize--) {
    /* different file */
    if (fn != ilist->file) {
      fn = ilist->file;
      /* close last if opened */
      ph_close(&fl, NULL);
      /* open new file */
      fl = ph_open(flist[fn].name, flist[fn].size, NULL);
    }
    /* if file exists */
    if (fl != -1) {
      /* read actual byte data from the file */
      for (i = 0; i < ilist->size; i++) {
        ilist->list[i].data = ph_read(fl, ilist->list[i].offs);
      }
    }
    /* next info item */
    ilist++;
  }
  /* close last file if any */
  ph_close(&fl, NULL);
}
